# -*- coding: utf-8 -*-
"""
Created on Sat Mar 23 13:09:07 2019

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import os
import sys
import inspect
import getpass
import requests

nutradir = os.path.join(os.path.expanduser("~"), ".nutra")

SERVER_HOST = "https://nutra-server.herokuapp.com"
try:
    SERVER_HOST = os.environ["NUTRA_SERVER_HOST"]
except:
    pass


def request(path, params):
    # print(f'{SERVER_HOST}/{path}')
    # print(params)
    return requests.get(url=f"{SERVER_HOST}/{path}", params=params)


def register(args=None):
    print("Register an online account!")
    username = input("Enter a username: ")
    email = input("Enter your email: ")
    password = getpass.getpass("Enter a password: ")
    confirm_password = getpass.getpass("Confirm password: ")

    params = dict(
        username=username,
        password=password,
        confirm_password=confirm_password,
        email=email,
    )

    response = request("register", params)
    print(response.json()["message"] + ": " + response.json()["data"])


def login(args=None):
    print("Login!")
    username = input("Enter your username: ")
    password = getpass.getpass("Enter your password: ")

    params = dict(username=username, password=password)

    response = request("login", params)
    token = response.json()["data"]
    print("Response: " + token)

    with open(f"{nutradir}/token", "a+") as token_file:
        token_file.write(token)


def main(args=None):
    if args == None:
        args = sys.argv[1:]

    # No arguments passed in
    if len(args) == 0:
        print(usage)

    # Otherwise we have some args
    for i, arg in enumerate(args):
        rarg = args[i:]
        if hasattr(cmdmthds, arg):
            getattr(cmdmthds, arg).mthd(rarg[1:])
            break
        # Activate method for opt commands, e.g. `-h' or `--help'
        elif altcmd(i, arg) != None:
            altcmd(i, arg)(rarg[1:])
            break
        # Otherwise we don't know the arg
        else:
            print(f"error: unknown option `{arg}'.  See 'nutra db --help'.")
            break


def altcmd(i, arg):
    for i in inspect.getmembers(cmdmthds):
        for i2 in inspect.getmembers(i[1]):
            if i2[0] == "altargs" and arg in i2[1]:
                return i[1].mthd
    return None


class cmdmthds:
    """ Where we keep the `cmd()` methods && opt args """

    class register:
        def mthd(rarg):
            register()

    class login:
        def mthd(rarg):
            login()

    class help:
        def mthd(rarg):
            print(usage)

        altargs = ["-h", "--help"]


usage = f"""nutra: Remote connection tool

Usage: nutra remote <command>

Commands:
    list | -l  list off databases stored on your computer"""

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Created on Mon Oct 29 23:44:06 2018

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
END NOTICE
"""

import os
import sys
import getpass
import inspect
from colorama import Style, Fore, Back, init


nutradir = f'{os.path.expanduser("~")}/.nutra'


def main(args=sys.argv):
    # os.chdir(os.path.expanduser("~"))
    # os.makedirs('.nutra/users', 0o755, True)

    if args == None:
        args = sys.argv

    # No arguments passed in
    if len(args) == 0:
        print(usage)

    # Otherwise we have some args
    # print(args)
    for i, arg in enumerate(args):
        rarg = args[i:]
        # Ignore first argument, as that is filename
        if arg == __file__:
            if len(args) == 1:
                print(usage)
                continue
            else:
                continue
        # Activate method for command, e.g. `help'
        elif hasattr(cmdmthds, arg):
            getattr(cmdmthds, arg).mthd(rarg[1:])
            break
        # Activate method for opt commands, e.g. `-h' or `--help'
        elif altcmd(i, arg) != None:
            altcmd(i, arg)(rarg[1:])
            break
        # Otherwise we don't know the arg
        print(f"error: unknown option `{arg}'.  See 'nutra config --help'.")
        break


def altcmd(i, arg):
    for i in inspect.getmembers(cmdmthds):
        for i2 in inspect.getmembers(i[1]):
            if i2[0] == "altargs" and arg in i2[1]:
                return i[1].mthd
    return None


class cmdmthds:
    """ Where we keep the `cmd()` methods && opt args """

    class new:
        altargs = ["--new", "-n"]

        def mthd(rarg):
            new_profile(rarg)

    class extra:
        altargs = ["-e"]

        def mthd(rarg):
            if len(rarg) == 0:
                print(extras)
            elif len(rarg) == 2:
                econfig(rarg)
            else:
                print("error: must specify only one option, and one value")

    class help:
        altargs = ["--help", "-h"]

        def mthd(rarg):
            print(usage)


def new_profile(rargs):
    """Creates a new profile, deletes old one."""
    name = getpass.getuser()
    gender = "n"
    age = 0
    print("Warning: This will create a new profile (log and db are kept)\n")
    # Name
    inpt = input(f"Enter name (blank for {name}): ")
    if inpt != "":
        name = inpt
    # Gender
    while True:
        inpt = input(f"Gender? [m/f/n]: ")
        if inpt == "m" or inpt == "f" or inpt == "n":
            gender = inpt
            break
    # Age
    while True:
        inpt = input(f"Age: ")
        try:
            inpt = int(inpt)
            if inpt > 0 and inpt < 130:
                age = inpt
                break
        except:
            pass
    # Write new profile
    os.makedirs(nutradir, 0o775, True)
    with open(f"{nutradir}/config.txt", "w+") as f:
        f.write(f"Name:{name}\n")
        f.write(f"Gender:{gender}\n")
        f.write(f"Age:{age}\n")
    print(
        "That's it for the basic config, you can see what more can be configured with `nutra config extras'"
    )


def econfig(rarg):
    """ Configures extra settings: weight, height, wrist size, and nutraent targets """
    print(f"option: {rarg[0]}")
    print(f"value:  {rarg[1]}")
    print("error: feature not implemented yet")


""" Usage commands """
usage = f"""Usage: nutra config <option> [<value>]

Options:
    new          create a new profile (log and db are kept)
    extra | -e   list extra options, or configure a specific one
"""

""" Extras usage"""
extras = f"""Usage: nutra config -e <option> [<value>]

Options:
    ht         height
    wt         weight
    wrist      wrist size"""

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Created on Sun Nov 11 23:57:03 2018

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import json

import requests
from tabulate import tabulate

from .utils.settings import SERVER_HOST


def cmd_analyze(args, unknown, arg_parser=None):
    # if args.r:
    #     print(f"recipe id: {args.r}")
    if not unknown:
        arg_parser.print_help()
        return

    # Get analysis
    food_ids = [int(x) for x in unknown]
    food_ids = ",".join([str(x) for x in food_ids])
    response = requests.get(f"{SERVER_HOST}/analyze", params={"food_ids": food_ids})
    analyses = response.json()["data"]
    # Get RDAs
    response = requests.get(f"{SERVER_HOST}/nutrients")
    rdas = response.json()["data"]
    rdas = {rda["id"]: rda for rda in rdas}

    # print(json.dumps(rdas))
    # print(json.dumps(analyses))
    # print(len(analyses))
    # print(rdas)

    for food in analyses:
        print(
            "\n======================================\n"
            f"==> {food['long_desc']} ({food['food_id']})\n"
            "======================================\n",
        )
        headers = ["nutrient", "amount", "units", "rda"]
        rows = []
        food_nutes = {x["nutr_id"]: x for x in food["nutrients"]}
        for id, nute in food_nutes.items():
            if not rdas[id]["rda"]:
                continue

            amount = food_nutes[id]["nutr_val"]
            if not amount:
                continue
            rda_ratio = round(amount / rdas[id]["rda"] * 100, 1)
            rows.append([nute["nutr_desc"], amount, rdas[id]["units"], f"{rda_ratio}%"])
        print(tabulate(rows, headers=headers, tablefmt="orgtbl"))
        # print(food["food_id"])


# def analyze(PK_No=None, grams=100):
#     dbs = db.fdbs()
#     for d in dbs:
#         for e in d.fdb_entries:
#             if e.pk_no == PK_No:
#                 print(f"Analyze: {e.foodname} ({grams}g)\n")

#                 # Prints basic fields
#                 max_basic_length = 0
#                 # TODO: this
#                 # for f in e.fields
#                 # Prints relative fields
#                 max_rel_length = 0
#                 for r in d.rels:
#                     for re in r.rel_entries:
#                         if re.pk_no == PK_No:
#                             if float(re.nutramt) == 0.0:
#                                 continue
#                             max_rel_length = (
#                                 max_rel_length
#                                 if max_rel_length > len(re.nutrname)
#                                 else len(re.nutrname)
#                             )
#                             # print(f'{re.nutrname} @ {re.nutramt}')
#                 for r in d.rels:
#                     for re in r.rel_entries:
#                         if re.pk_no == PK_No:
#                             reamt = round(float(re.nutramt) * grams / 100, 2)
#                             if reamt == 0.0:
#                                 continue
#                             pad_rel_name = re.nutrname + " " * (
#                                 max_rel_length - len(re.nutrname)
#                             )
#                             print(f"{pad_rel_name}  {reamt} mg")


# def main(args=None):
#     if args == None:
#         args = sys.argv

#     # No arguments passed in
#     if len(args) == 0:
#         print(usage)

#     # Otherwise we have some args
#     for i, arg in enumerate(args):
#         rarg = args[i:]
#         if hasattr(cmdmthds, arg):
#             getattr(cmdmthds, arg).mthd(rarg[1:])
#             break
#         # Activate method for opt commands, e.g. `-h' or `--help'
#         elif altcmd(i, arg) != None:
#             altcmd(i, arg)(rarg[1:])
#             break
#         # Otherwise we don't know the arg
#         else:
#             print(f"error: unknown option `{arg}'.  See 'nutra anl --help'.")
#             break


# def altcmd(i, arg):
#     for i in inspect.getmembers(cmdmthds):
#         for i2 in inspect.getmembers(i[1]):
#             if i2[0] == "altargs" and arg in i2[1]:
#                 return i[1].mthd
#     return None


# class cmdmthds:
#     """ Where we keep the `cmd()` methods && opt args """

#     class dbno:
#         def mthd(rarg):
#             if len(rarg) != 1 and len(rarg) != 2:
#                 exit(
#                     f"error: must specify only one PK_No, optionally followed by an amount in grams"
#                 )
#             try:
#                 PK_No = int(rarg[0])
#             except:
#                 exit("error: invalid integer number for PK_No")
#             try:
#                 grams = int(rarg[1])
#             except:
#                 print("warn: no grams provided, using 100g")
#                 grams = 100
#             analyze(PK_No, grams)

#         altargs = ["-n"]

#     class help:
#         def mthd(rarg):
#             print(usage)

#         altargs = ["-h", "--help"]


# usage = f"""Analyze ingredients, foods, recipes, and days.
# Version {version}

# Usage: nutra anl <option> [<value>]

# Options:
#     dbno | -n  analyze an ingredient by PK_No (NDB_No)"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Oct 26 06:38:40 2018

@author: shane
NOTICE
    This file is part of nutri, a nutrient analysis program.
        https://github.com/gamesguru/nutri
        https://pypi.org/project/nutri/

    nutri is an extensible nutrient analysis and composition application.
    Copyright (C) 2018  Shane Jaroch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
END NOTICE
"""

from colorama import Style, Fore

# Colors
critlevel = 0.3
critcolor = Fore.RED

warnlevel = 0.7
warncolor = Fore.YELLOW

safecolor = Fore.CYAN

overlevel = 1.3
overcolor = Fore.LIGHTBLACK_EX
# End colors


def color(perc):
    if perc <= critlevel:
        return critcolor
    elif perc > critlevel and perc <= warnlevel:
        return warncolor
    elif perc > warnlevel and perc < overlevel:
        return safecolor
    else:
        return overcolor


def progbar(ing=5, rda=100, buf=25):
    perc = ing / rda
    ticks = round(perc * buf)
    ticks = ticks if ticks < buf else buf
    bar = "<"
    for i in range(ticks):
        bar += "="
    for i in range(buf - ticks):
        bar += " "
    bar += ">"
    c = color(perc)
    p = fmtperc(perc)
    fstr = f"{c}{bar} {p}{Style.RESET_ALL}"
    return fstr


def fmtperc(perc):
    p = str(perc * 100) + "%"
    for i in range(len(p), 6):
        p = f" {p}"
    return f"{color(perc)}{p}{Style.RESET_ALL}"


for i in range(0, 175, 25):
    print(progbar(i))



# -*- coding: utf-8 -*-
"""
Created on Sat Mar 23 13:09:07 2019

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import os

from dotenv import load_dotenv

# Read in .env file if it exists locally, else look to env vars
load_dotenv()

NUTRA_DIR = os.path.join(os.path.expanduser("~"), ".nutra")

REMOTE_HOST = "https://nutra-server.herokuapp.com"
SERVER_HOST = os.getenv("NUTRA_OVERRIDE_LOCAL_SERVER_HOST", REMOTE_HOST)

TESTING = SERVER_HOST != REMOTE_HOST



# -*- coding: utf-8 -*-
"""
Created on Sat Feb  1 00:31:44 2020

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


def cmd_account(args, unknown):
    _args = vars(args)
    if len(unknown) < 1:
        # Print help if no args
        # args.print_help()
        return


def cmd_login(args, unknown, arg_parser=None):
    pass



# -*- coding: utf-8 -*-
"""
Created on Fri Apr 12 19:45:43 2019

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import sys
import shutil
import inspect
from libnutra import remote
from tabulate import tabulate


def rank(type, rargs):
    words = rargs[1:]

    if type == "nutr_no":
        nutr_no = int(rargs[0])
        params = dict(nutr_no=nutr_no)
        response = remote.request("sort", params=params)
        results = response.json()["data"]
    else:  # tagname
        tagname = rargs[0]
        params = dict(tagname=tagname)
        response = remote.request("sort", params=params)
        results = response.json()["data"]

    print_id_and_long_desc_and_nutr_val(results)


def print_id_and_long_desc_and_nutr_val(results):
    # Current terminal size
    bufferwidth = shutil.get_terminal_size()[0]
    bufferheight = shutil.get_terminal_size()[1]

    rows = []
    for i, r in enumerate(results[0]["foods"]):
        if i == bufferheight - 4:
            break
        food_id = str(r["food_id"])
        food_name = str(r["long_desc"])
        nutr_val = str(r["nutr_val"])
        avail_buffer = bufferwidth - len(food_id) - len(nutr_val) - 25
        if len(food_name) > avail_buffer:
            rows.append([food_id, food_name[:avail_buffer] + "...", nutr_val])
        else:
            rows.append([food_id, food_name, nutr_val])
    print(
        tabulate(rows, headers=["food_id", "food_name", "nutr_val"], tablefmt="orgtbl")
    )


def main(args=None):
    if args == None:
        args = sys.argv[1:]

    # No arguments passed in
    if len(args) == 0:
        print(usage)

    # Otherwise we have some args
    for i, arg in enumerate(args):
        rarg = args[i:]
        if hasattr(cmdmthds, arg):
            getattr(cmdmthds, arg).mthd(rarg[1:])
            break
        # Activate method for opt commands, e.g. `-h' or `--help'
        elif altcmd(i, arg) != None:
            altcmd(i, arg)(rarg[1:])
            break
        # Otherwise we don't know the arg
        else:
            print(f"error: unknown option `{arg}'.  See 'nutra rank --help'.")
            break


def altcmd(i, arg):
    for i in inspect.getmembers(cmdmthds):
        for i2 in inspect.getmembers(i[1]):
            if i2[0] == "altargs" and arg in i2[1]:
                return i[1].mthd
    return None


class cmdmthds:
    """ Where we keep the `cmd()` methods && opt args """

    class nutrno:
        def mthd(rarg):
            rank("nutr_no", rarg)

        altargs = ["-n"]

    class tagname:
        def mthd(rarg):
            rank("tagname", rarg)

        altargs = ["-t"]

    class help:
        def mthd(rarg):
            print(usage)

        altargs = ["-h", "--help"]


usage = f"""nutra: Rank foods by Nutr_No or Tagname

Usage: nutra search <flags> <query>

Flags:
    -n            search by Nutr_No
    -t            search by Tagname
    -u            filter USDA only
    -b            filter BFDB only
    -n            filter nutra DB only
    -nub          search all three DBs
    --help | -h   print help"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Sep 30 08:39:42 2018

@author: shane
NOTICE
    This file is part of nutri, a nutrient analysis program.
        https://github.com/gamesguru/nutri
        https://pypi.org/project/nutri/

    nutri is an extensible nutrient analysis and composition application.
    Copyright (C) 2018  Shane Jaroch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
END NOTICE
"""

import os
import datetime


class Nutrient:
    def __init__(self, field, friendlyname=None, rda=None, units=None):
        self.field = field
        self.friendlyname = field if friendlyname is None else friendlyname
        self.rda = rda
        self.units = units
        nutrients.append(self)


nutrients = []
Nutrient("Calories", 2000, "cals")
Nutrient("Thiamin", 0.9, "mg")


class Food:
    def __init__(
        self, db, key, name, Nutrients=None, per_units=None
    ):  # fields? recipes? no. no.no
        self.db = db
        self.key = key
        self.name = name
        self.Nutrients = [] if Nutrients is None else Nutrients
        self.per_units = [] if per_units is None else per_units


class Recipe:
    def __init__(self, name, foods=None):
        self.name = name
        self.foods = [] if foods is None else foods


class Meal:
    def __init__(self, name, time=None, foods=None, recipes=None):
        self.name = name
        self.time = datetime.datetime.now() if time is None else time
        self.foods = [] if foods is None else foods
        self.recipes = [] if recipes is None else recipes


class Day:
    def __init__(self, date, meals=None):
        self.date = date
        meals = [] if meals is None else meals


# def main():
#     pass


# if __name__ == '__main__':
#     main()



# -*- coding: utf-8 -*-
"""
Created on Sat Oct 27 20:28:06 2018

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutraent analysis and composition application.
Copyright (C) 2018  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import sys
import shutil
import inspect
from libnutra import remote
from tabulate import tabulate


def cmd_search(args, unknown, arg_parser=None):
    return search(words=unknown)


def print_id_and_long_desc(results):
    # Current terminal size
    bufferwidth = shutil.get_terminal_size()[0]
    bufferheight = shutil.get_terminal_size()[1]

    rows = []
    for i, r in enumerate(results):
        if i == bufferheight - 4:
            break
        food_id = str(r["food_id"])
        food_name = str(r["long_desc"])
        avail_buffer = bufferwidth - len(food_id) - 15
        if len(food_name) > avail_buffer:
            rows.append([food_id, food_name[:avail_buffer] + "..."])
        else:
            rows.append([food_id, food_name])
    print(tabulate(rows, headers=["food_id", "food_name"], tablefmt="orgtbl"))


def search(words, dbs=None):
    """ Searches all dbs, foods, recipes, recents and favorites. """
    params = dict(terms=",".join(words))

    response = remote.request("search", params=params)
    results = response.json()["data"]

    print_id_and_long_desc(results)


def main(args=None):
    if args == None:
        args = sys.argv[1:]

    words = []

    # No arguments passed in
    if len(args) == 0:
        print(usage)

    # Otherwise we have some args
    for i, arg in enumerate(args):
        rarg = args[i:]
        if hasattr(cmdmthds, arg):
            getattr(cmdmthds, arg).mthd(rarg[1:])
            if arg == "help":
                break
        # Activate method for opt commands, e.g. `-h' or `--help'
        elif altcmd(i, arg) != None:
            altcmd(i, arg)(rarg[1:])
            if arg == "-h" or arg == "--help":
                break
        # Otherwise we don't know the arg
        else:
            words.append(arg)

    if len(words) > 0:
        search(words)


def altcmd(i, arg):
    for i in inspect.getmembers(cmdmthds):
        for i2 in inspect.getmembers(i[1]):
            if i2[0] == "altargs" and arg in i2[1]:
                return i[1].mthd
    return None


class cmdmthds:
    """ Where we keep the `cmd()` methods && opt args """

    class help:
        def mthd(rarg):
            print(usage)

        altargs = ["-h", "--help"]

    class rank:
        def mthd(rarg):
            rank(rarg)

        altargs = ["-r", "--rank"]


usage = f"""nutra: Search tool

Usage: nutra search <flags> <query>

Flags:
    -r            rank foods by Nutr_No or Tagname
    -u            search USDA only
    -b            search BFDB only
    -n            search nutra DB only
    -nub          search all three DBs
    --help | -h   print help"""

if __name__ == "__main__":
    main()
